#!/usr/bin/perl

use strict;
use warnings;
use Image::Magick;

#use GD;
#use GD::Text::Wrap;

my %system;

#基本キャプチャファイル名。
$system{ 'srcfile' }   = 'temp';
#拡張子。
$system{ 'tail' }      = 'png';
#カメラの最大数。
$system{ 'max' }       = 1;
#大きな画像のサイズ。
$system{ 'pic_w' }     = 400;
$system{ 'pic_h' }     = 300;
#タイムスタンプのフォントサイズ。
$system{ 'fontsize' }  = 12;
#タイムスタンプの文字色。
$system{ 'fontcolor' } = 0xff00ff;
#テキストボックスの横幅。
$system{ 'text_w' }    = 390;
#テキストボックスの位置。
$system{ 'text_x' }    = 0;
$system{ 'text_y' }    = 270;
#サムネイルのサイズ。
$system{ 'thumb_w' }   = 80;
$system{ 'thumb_h' }   = 60;

my $n;
for( $n = 0 ; $n < $system{ 'max' } ; ++$n )
{
  #時間の取得。
  my ( $sec, $min, $hour, $day, $month, $year ) = localtime( time() );

  #撮影時間を記録。
  my $time = sprintf( '%d/%02d/%02d %02d:%02d:%02d', $year + 1900, $month + 1, $day, $hour, $min, $sec );

  #ディレクトリの名前を作る。
  my $dirname = sprintf( './log_%d/%d%02d%02d', $n, $year + 1900, $month + 1, $day );

  #基本ファイル名を作る。
  my $filename = sprintf( '%02d%02d%02d', $hour, $min, $sec );

  #存在しない場合は作る。
  unless( -d $dirname )
  {
    mkdir( $dirname, 0705 );
    mkdir( $dirname.'/pic', 0705 );
    mkdir( $dirname.'/thumb', 0705 );
  }

  #画像ファイルからイメージを作る。
#  my $gd = GD::Image->new( sprintf( '%s_%d.%d', $system{ 'srcfile' }, $n, $system{ 'tail' } ) );
  my $image = Image::Magick->new;
  $image->Read( sprintf( '%s_%d.%d', $system{ 'srcfile' }, $n, $system{ 'tail' } ) );

  #大きな画像を作る。
#  my $pic = &CopyImage( $gd, $system{ 'pic_w' }, $system{ 'pic_h' } );
  my $pic = &CopyImage( $image, $system{ 'pic_w' }, $system{ 'pic_h' } );

  #日付を右下に描画する。
  &TimeStamp( $pic, $time );

  #大きな画像を出力。
  my $pic_file = sprintf( '%s/pic/%s.png', $dirname, $filename );
  &CreateImage( $pic_file, $pic );

  #サムネイルを作る。
  my $thumb = &CopyImage( $gd, $system{ 'thumb_w' }, $system{ 'thumb_h' } );

  #サムネイルを出力。
  my $thumb_file = sprintf( '%s/thumb/%s.png', $dirname, $filename );
  &CreateImage( $thumb_file, $thumb );

}

sub CopyImage( $$$$ )
{
#  my ( $gd, $cw, $ch ) = @_;
  my ( $image, $cw, $ch ) = @_;

  #元画像のサイズを取得する。
#  my ( $width, $height ) = $gd->getBounds;
  my ( $width, $height ) = $img->Get( 'width', 'height' );

  #画像オブジェクトを生成する。
#  my $image = GD::Image->new( $cw, $ch, 1);
  #元画像を縮小して張りつける。
#  $image->copyResampled( $gd, 0, 0, 0, 0, $cw, $ch, $width, $height );
  $image->Resize( width => $ch, height => $ch );
  $image->Crop( width => $cw, height => $ch, x => 0, y => 0 );
  return $image;
}

sub CreateImage( $$ )
{
  my ( $filename, $gd ) = @_;
  #ファイルをバイナリモードで開いてPNG形式の画像として吐く。
  if( open ( IMAGE, ">$filename" ) )
  {
    binmode ( IMAGE );
    print IMAGE $gd->png;
    close ( IMAGE );
    return 0;
  }
  return 1;
}

sub TimeStamp( $$ )
{
  my ( $gd, $time ) = @_;

  #テキストボックスの生成。
  my $wrapbox = GD::Text::Wrap->new( $gd, color => $system{ 'fontcolor' },
                                     text => $time, width => $system{ 'text_w' } );
  #フォントの種類を設定。
  $wrapbox->set_font( gdMediumBoldFont );
  #フォントのサイズを設定。
  $wrapbox->set_font( 'arial', $system{ 'fontsize' } );
  #文字の開始場所を設定。
  $wrapbox->set( align => 'right' );
  #テキストボックスの配置位置を設定して書き込む。
  $wrapbox->draw( $system{ 'text_x' }, $system{ 'text_y' } );

  return $wrapbox;
}
